{

  This demo requires TRichView 17.3 or newer.


  This example shows how to convert HTML to RTF files using TRichView and
  HTMLViewer.

  HTMLViewer: https://github.com/BerndGabriel/HtmlViewer
  TRichView: http://www.trichview.com

  If HTMLViewer is installed before TRichView, TRichView Setup installs
  TrvHtmlViewImporter automatically.


  Currently, HTMLViewer does not have packages for Delphi 10.1 and 10.2.
  You can make them from the packages for Delphi 10:
  - open FrameViewerXE9.dproj and save it
    as FrameViewerXE10.dproj (for 10.1) or
    as FrameViewerXE11.dproj (for 10.2)
  - open dclFrameViewerXE9.dproj and save it
    as dclFrameViewerXE10.dproj (for 10.1) or
    as dclFrameViewerXE11.dproj (for 10.2)

}


unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, Dialogs, ComCtrls, StdCtrls,
  // TRichView units
  RVStyle, RVScroll, RichView, RVFile, RVGrHandler, RVGifAnimate2007,
  RVTypes,
  // Indy units
  IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient, IdHTTP,
  // HTML viewer and importer units
  rvHtmlViewImport, HTMLUn2, HtmlView, IdIOHandler, IdIOHandlerSocket,
  IdIOHandlerStack, IdSSL, IdSSLOpenSSL;

type
  TfrmHTMLToRTF = class(TForm)
    RichView1: TRichView;
    RVStyle1: TRVStyle;
    HtmlViewer1: THtmlViewer;
    Button1: TButton;
    RVHTMLViewImporter1: TRVHTMLViewImporter;
    IdHTTP1: TIdHTTP;
    StatusBar1: TStatusBar;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    IdSSLIOHandlerSocketOpenSSL1: TIdSSLIOHandlerSocketOpenSSL;
    procedure Button1Click(Sender: TObject);
    procedure RichView1ImportPicture(Sender: TCustomRichView;
      const Location: TRVUnicodeString; Width, Height: Integer; var Graphic: TGraphic);
    procedure HtmlViewer1htStreamRequest(Sender: TObject; const SRC: String;
      var Stream: TStream);
  private
    { Private declarations }
    procedure ShowMessage(const S: String);
  public
    { Public declarations }
  end;

var
  frmHTMLToRTF: TfrmHTMLToRTF;

implementation

{$R *.dfm}

procedure TfrmHTMLToRTF.ShowMessage(const S: String);
begin
  StatusBar1.SimpleText := S;
end;

procedure TfrmHTMLToRTF.Button1Click(Sender: TObject);
begin
  if OpenDialog1.Execute and SaveDialog1.Execute then
  begin
    ShowMessage('Loading HTML file...');
    HTMLViewer1.LoadFromFile(OpenDialog1.FileName);
    ShowMessage('Converting...');
    RVHTMLViewImporter1.ImportHtmlViewer(HTMLViewer1, RichView1,
      ExtractFilePath(OpenDialog1.FileName));
    RichView1.RVData.NormalizeDocument(0, True);
    ShowMessage('Preparing for export...');
    RichView1.Format; // necessary for documents containing tables
    ShowMessage('Saving RTF file...');
    RichView1.SaveRTF(SaveDialog1.FileName, False);
    ShowMessage('');
  end;
end;

// Downloading external content is optional

// Downloading external pictures
procedure TfrmHTMLToRTF.RichView1ImportPicture(Sender: TCustomRichView;
  const Location: TRVUnicodeString; Width, Height: Integer; var Graphic: TGraphic);
var
  Stream: TMemoryStream;
begin
  // This is the most simple implementation of downloading external picture
  // In real applications, you should use some in-memory cache to prevent
  // downloading the same picture many times
  if (CompareText(Copy(Location, 1, 7), 'http://') = 0) or
     (CompareText(Copy(Location, 1, 8), 'https://') = 0) then
  begin
    Stream := TMemoryStream.Create;
    try
      IdHTTP1.Get(Location, Stream);
      Stream.Position := 0;
      Graphic := RVGraphicHandler.LoadFromStream(Stream);
    except
    end;
  end;
end;

// Downloading other external files, such as CSS
procedure TfrmHTMLToRTF.HtmlViewer1htStreamRequest(Sender: TObject;
  const SRC: string; var Stream: TStream);
var
  URL: String;
begin
  URL := RVHTMLViewImporter1.GetFullFileName(SRC);
  if (CompareText(Copy(URL, 1, 7), 'http://') = 0) or
     (CompareText(Copy(URL, 1, 8), 'https://') = 0) then
  begin
    Stream := TMemoryStream.Create;
    try
      IdHTTP1.Get(URL, Stream);
    except
      Stream.Free;
      Stream := nil;
    end;
  end;
end;


end.
